{ File   : ConvertSomething.dpr
  Author : Deniz Oezmen
  Created: 2006-09-15
  Changed: 2006-09-15

  Converts something to something else. :-)
}
program ConvertSomething;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils, Math;

const
  StringAttrUndef: AnsiString  = '';
  FloatAttrUndef : Double      = -5.0E-324;

type
  TSct = record
           Name    : AnsiString;
           X, Y,
           Rotation: Double
         end;

var
  InFile,
  OutFile: TextFile;

function OpenInFile(var InFile: TextFile;
  const InFileName: AnsiString): Boolean;
begin
  FileMode := fmOpenRead;

  AssignFile(InFile, InFileName);
  {$I-}
  Reset(InFile);
  {$I+}
  Result := (IOResult = 0)
end;

function OpenOutFile(var OutFile: TextFile;
  const OutFileName: AnsiString): Boolean;
begin
  FileMode := fmShareDenyWrite;

  AssignFile(OutFile, OutFileName);
  {$I-}
  Rewrite(OutFile);
  {$I+}
  Result := (IOResult = 0)
end;

function IsEmpty(const s: AnsiString): Boolean;
var
  i: Integer;
begin
  Result := True;

  for i := 1 to Length(s) do
    Result := Result and (s[i] in [#009, ' '])
end;

procedure InitSctData(out Sct: TSct);
begin
  with Sct do
  begin
    Name := StringAttrUndef;
    X := FloatAttrUndef;
    Y := FloatAttrUndef;
    Rotation := FloatAttrUndef
  end
end;

procedure CheckStringAttr(const Attr, AttrName: AnsiString; SctCnt: Longint);
begin
  if Attr = StringAttrUndef then
    WriteLn(Format('Warning: %s has not been defined in object %d.',
      [AttrName, SctCnt]))
end;

procedure CheckFloatAttr(Attr: Double; const AttrName: AnsiString;
  SctCnt: Longint);
begin
  if Attr = FloatAttrUndef then
    WriteLn(Format('Warning: %s has not been defined in object %d.',
      [AttrName, SctCnt]))
end;

procedure WriteSctData(const OutFile: TextFile; SctCnt: Longint; var Sct: TSct);
begin
  with Sct do
  begin
    // check data
    CheckStringAttr(Name, 'Name', SctCnt);
    CheckFloatAttr(X, 'X', SctCnt);
    CheckFloatAttr(Y, 'Y', SctCnt);
    CheckFloatAttr(Rotation, 'Rotation', SctCnt);

    // write data
    WriteLn(OutFile, Format('Object%.3d.TypeName=%s', [SctCnt, Name]));
    WriteLn(OutFile, Format('Object%.3d.MapClass=0', [SctCnt]));
    WriteLn(OutFile, Format('Object%.3d.Chance=100', [SctCnt]));
    WriteLn(OutFile, Format('Object%.3d.PositionX=%.6f', [SctCnt, X]));
    WriteLn(OutFile, Format('Object%.3d.PositionY=%.6f', [SctCnt, Y]));
    WriteLn(OutFile, Format('Object%.3d.Angle=%.6f', [SctCnt, Rotation]))
  end
end;

procedure ConvertStringAttr(const Content: AnsiString; var Attr: AnsiString;
  const AttrName: string; SctCnt, LineCnt: Longint);
begin
  if Attr <> StringAttrUndef then
    WriteLn(Format('Warning: %s redefined in object %d, line %d.',
      [AttrName, SctCnt, LineCnt]));
  Attr := Content
end;

procedure ConvertFloatAttr(const Content: AnsiString; var Attr: Double;
  const AttrName: string; SctCnt, LineCnt: Longint);
begin
  if Attr <> FloatAttrUndef then
    WriteLn(Format('Warning: %s redefined in object %d, line %d.',
      [AttrName, SctCnt, LineCnt]));
  if not TryStrToFloat(Content, Attr) then
    WriteLn(Format('Warning: expected float in line %d, found ''%s''.',
      [LineCnt, Content]))
end;

procedure Convert(var InFile, OutFile: TextFile; const TextureName: AnsiString);
var
  Line,
  Ident,
  Content: AnsiString;
  SctCnt,
  LineCnt,
  SubPos : Integer;
  Sct    : TSct;
begin
  DecimalSeparator := '.';

  // first pass: count object sections
  SctCnt := 0;

  while not Eof(InFile) do
  begin
    ReadLn(InFile, Line);

    SubPos := Pos('[macro]', AnsiLowerCase(Line));
    if SubPos > 0 then
      if IsEmpty(LeftStr(Line, SubPos - 1)) and
        IsEmpty(RightStr(Line, Length(Line) - SubPos - 6)) then
        Inc(SctCnt)
  end;

  if SctCnt = 0 then
  begin
    WriteLn('Error: Nothing to do ...');
    Exit
  end;

  // write output header
  WriteLn(OutFile, Format('[%s]', [TextureName]));
  WriteLn(OutFile, Format('ObjectCount=%d', [SctCnt]));

  // second pass: gather data
  Reset(InFile);

  SctCnt := -1;
  LineCnt := 0;

  while not Eof(InFile) do
  begin
    ReadLn(InFile, Line);
    Inc(LineCnt);

    // match line againt known inputs
    if IsEmpty(Line) then
      Continue;

    SubPos := Pos('[macro]', AnsiLowerCase(Line));
    if SubPos > 0 then
      // new object?
      if IsEmpty(LeftStr(Line, SubPos - 1)) and
        IsEmpty(RightStr(Line, Length(Line) - SubPos - 6)) then
      begin
        // if this is not the first entry, write the previous one
        if SctCnt <> -1 then
          WriteSctData(OutFile, SctCnt, Sct);

        InitSctData(Sct);

        Inc(SctCnt);
        Continue
      end;

    SubPos := Pos('=', Line);
    if SubPos > 0 then
    begin
      // split input line
      Ident := AnsiLowerCase(LeftStr(Line, SubPos - 1));
      Content := RightStr(Line, Length(Line) - SubPos);

      // process known attributes
      if Ident = 'name' then
        ConvertStringAttr(Content, Sct.Name, 'Name', SctCnt, LineCnt)
      else if Ident = 'x' then
        ConvertFloatAttr(Content, Sct.X, 'X', SctCnt, LineCnt)
      else if Ident = 'y' then
        ConvertFloatAttr(Content, Sct.Y, 'Y', SctCnt, LineCnt)
      else if Ident = 'rotation' then
        ConvertFloatAttr(Content, Sct.Rotation, 'Rotation', SctCnt, LineCnt)
      // ignore all other definitions
    end
    else
      WriteLn(Format('Warning: Unknown data in line %d', [LineCnt]))
  end;

  // write final data set
  WriteSctData(OutFile, SctCnt, Sct);

  WriteLn(Format('Done: %d objects converted.', [SctCnt + 1]))
end;

begin
  if ParamCount < 3 then
  begin
    WriteLn(Format('Usage: %s <Input.txt> <TextureName.bmp> <Output.txt>',
      [ExtractFileName(ParamStr(0))]));
    Exit
  end;

  if not OpenInFile(InFile, ParamStr(1)) then
  begin
    WriteLn(Format('Error: Could not open input file %s.', [ParamStr(1)]));
    Exit
  end;

  if not OpenOutFile(OutFile, ParamStr(3)) then
  begin
    CloseFile(InFile);

    WriteLn(Format('Error: Could not open output file %s.', [ParamStr(3)]));
    Exit
  end;

  WriteLn(Format('Converting %s to %s with texture name %s.',
    [ParamStr(1), ParamStr(3), ParamStr(2)]));

  Convert(InFile, OutFile, ParamStr(2));

  CloseFile(InFile);
  CloseFile(OutFile)
end.
